/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */


import org.glite.ce.monitor.queryprocessor.ClassAdProcessor;
import org.glite.ce.monitorapij.queryprocessor.QueryResult;
import org.glite.ce.monitorapij.sensor.SensorException;
import org.glite.ce.monitorapij.types.Dialect;
import org.glite.ce.monitorapij.types.Query;
import org.glite.ce.monitorapij.types.Topic;

import junit.framework.TestCase;


public class ClassAdProcessorTest extends TestCase {
    
    private ClassAdProcessor classAdProcessor = null;

    
	
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
	    classAdProcessor = new ClassAdProcessor();
	}

	public void testClassAdProcessor() {
	    assertNotNull(classAdProcessor);
	}

	public void testEvaluate() {
	    Topic topic = new Topic();
		Dialect[] dialects = new Dialect[2];
	      topic.setName("TEST");
	      Dialect dialect = new Dialect();
			dialect.setName("Test dialect");
			dialects[0]=dialect;
			dialect.setName("ISM_LDIF");
			dialects[1]=dialect;
			topic.setDialect(dialects);
	      TestSensor testSensor = new TestSensor();
	      testSensor.setName("Test Sensor");
	      testSensor.addFormat(new TestFormat("ISM_LDIF"));
	      try {
			testSensor.init();
		} catch (SensorException e2) {
			e2.printStackTrace();
		}
		testSensor.setType("Test Sensor Type");
		// il sensor Type diventa il topic name
		TestSensorEvent testSensorEvent = new TestSensorEvent( testSensor, System.currentTimeMillis( ), new String[]{"Test Event"} );
		String[] message = new String[1];
		message[0] = "this is a test";
		testSensorEvent.setMessage(message);
		try {
            testSensorEvent.applyFormat(new TestFormat("Test Dialect"));
//            testSensorEvent.applyFormat(new TestFormat("ISM_LDIF"));
        } catch (SensorException e1) {
            e1.printStackTrace();
        }
		Query query = new Query();
		QueryResult[] queryResults =null; 
		// case query null
		String exception = "";
		try {
		    queryResults = classAdProcessor.evaluate(null, testSensorEvent);
        } catch (Exception e) {
            exception = e.getMessage();
        }
        assertEquals("Invalid argument: the query is null!",exception);
        
        // query Language null
        try {
		    queryResults = classAdProcessor.evaluate(query, testSensorEvent);
        } catch (Exception e) {
            exception = e.getMessage();
        }
        assertEquals("the query language is null",exception);

        // query language differret from ClassAd
	      query.setQueryLanguage("nothing");
	        try {
			    queryResults = classAdProcessor.evaluate(query, testSensorEvent);
	        } catch (Exception e) {
	            exception = e.getMessage();
	        }
	        assertEquals("Query Language mismatch: expected \"ClassAd\", found \"nothing\"",exception);

		      query.setQueryLanguage("ClassAd");
	        // null expression
		        try {
				    queryResults = classAdProcessor.evaluate(query, testSensorEvent);
		        } catch (Exception e) {
		            exception = e.getMessage();
		        }
		        assertEquals("the query expression is null",exception);
		   
		        query.setExpression("test");
        // no exceptions
		try {
		    queryResults = classAdProcessor.evaluate(query, testSensorEvent);
        } catch (Exception e) {
            e.printStackTrace();
        }
        assertNotNull(queryResults);
        
//
//		try {
//            testSensorEvent.applyFormat(new TestFormat("ISM_LDIF"));
//        } catch (SensorException e1) {
//            e1.printStackTrace();
//        }
//        try {
//		    queryResults = classAdProcessor.evaluate(query, testSensorEvent);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        assertNotNull(queryResults);
	}

}
