/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.rpc.handler.MessageContext;

import org.glite.ce.commonj.utils.JarClassLoader;

import junit.framework.TestCase;
/*
 * Copyright (c) 2004 on behalf of the EU EGEE Project:
 * The European Organization for Nuclear Research (CERN),
 * Istituto Nazionale di Fisica Nucleare (INFN), Italy
 * Datamat Spa, Italy
 * Centre National de la Recherche Scientifique (CNRS), France
 * CS Systeme d'Information (CSSI), France
 * Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 * Universiteit van Amsterdam (UvA), Netherlands
 * University of Helsinki (UH.HIP), Finland
 * University of Bergen (UiB), Norway
 * Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 * 
 * Authors: Antonino Stefano Borgia (stefano.borgia@pd.infn.it)
 */

/**
 * @author borgia
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class JarClassLoaderTest extends TestCase {
    private JarClassLoader jarClassLoader = null;
    private String href = "";
    private URL hrefURL = null;
    private MessageContext messageContext = null;

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        TestUtilities tu = new TestUtilities();
        messageContext = tu.getMessageContext();
        href = (String) messageContext.getProperty("sensorURI");
        hrefURL = new URL(href+"/TestSensor.jar");
        jarClassLoader = new JarClassLoader(href,this.getClass().getClassLoader());
    }

    /*
     * Class under test for void JarClassLoader(String, ClassLoader)
     */
    public void testJarClassLoaderStringClassLoader() {
        assertNotNull(jarClassLoader);
    }

    /*
     * Class under test for void JarClassLoader(URL, ClassLoader)
     */
    public void testJarClassLoaderURLClassLoader() {
    }

    /*
     * Class under test for void addJarURL(URL)
     */
    public void testAddJarURLString() {
        //href null:
        String nullHref = null;

        try {
            jarClassLoader.addJarURL(nullHref);
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
        try {
            jarClassLoader.addJarURL(href+"/config");
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
        URL[] urls = jarClassLoader.getURLs();
            try {
                assertEquals(new URL(href+"/config"),urls[1]);
            } catch (MalformedURLException e1) {
                e1.printStackTrace();
            }
    }

    /*
     * Class under test for void addJarURL(String)
     */
    public void testAddJarURLURL() {
        URL url = null;
        try {
            url = new URL(href+"/config");
            jarClassLoader.addJarURL(url);
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
        URL[] urls = jarClassLoader.getURLs();
        assertEquals(url,urls[1]);
    }

    public void testGetMainClassName() {
        String mainClassName = ""; 
        try {
            mainClassName = jarClassLoader.getMainClassName(hrefURL);
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        assertEquals("TestSensor",mainClassName);
    }

    public void testInvokeClass() {
        String[] args = new String[1];
        String exception = "";
        try {
            jarClassLoader.invokeClass("TestSensor",args);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            exception = e.getLocalizedMessage();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
        // waiting to decide if this exception is acceptable
        assertEquals("TestSensor.main([Ljava.lang.String;)",exception);
        
    }

}
