/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */

import java.io.FileNotFoundException;
import java.net.URI;
import java.net.URISyntaxException;

import org.apache.axis.MessageContext;
import org.apache.axis.types.URI.MalformedURIException;
import org.glite.ce.monitor.SubscriptionPersistent;
import org.glite.ce.monitor.holder.ActionHolder;
import org.glite.ce.monitor.holder.NotificationHolder;
import org.glite.ce.monitor.holder.QueryProcessorHolder;
import org.glite.ce.monitor.holder.TopicHolder;
import org.glite.ce.monitor.queryprocessor.ClassAdProcessor;
import org.glite.ce.monitor.registry.SubscriptionRegistry;
import org.glite.ce.monitorapij.queryprocessor.QueryProcessor;
import org.glite.ce.monitorapij.sensor.SensorEvent;
import org.glite.ce.monitorapij.sensor.SensorException;
import org.glite.ce.monitorapij.types.Dialect;
import org.glite.ce.monitorapij.types.Policy;
import org.glite.ce.monitorapij.types.Query;
import org.glite.ce.monitorapij.types.Topic;

import junit.framework.TestCase;

public class NotificationHolderTest extends TestCase {

	
	private NotificationHolder notificationHolder = null;
	private MessageContext messageContext = null;
	private TopicHolder topicHolder = null;
	private SubscriptionRegistry  subscriptionRegistry = null;
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {

	   	TestUtilities testUtilities = new TestUtilities();
	   	messageContext= testUtilities.getMessageContext();
	   	topicHolder = new TopicHolder();
	   	try {
//			String subscriptionFile = (String)messageContext.getProperty("sensorURI")+"/subscription.reg";
			subscriptionRegistry = new SubscriptionRegistry();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		notificationHolder = new NotificationHolder(1000,topicHolder,subscriptionRegistry);
//	   	notificaMessageContext messageContexttionHolder = new NotificationHolder(1000,topicHolder);
	}

	/*
	 * Class under test for void NotificationHolder(int, TopicHolder)
	 */
	public void testNotificationHolderintTopicHolder() {
		notificationHolder = new NotificationHolder(1000,topicHolder);
		assertNotNull(notificationHolder);
	}

	/*
	 * Class under test for void NotificationHolder(int, TopicHolder, SubscriptionRegistry)
	 */
	public void testNotificationHolderintTopicHolderSubscriptionRegistry() {
		
		assertNotNull(notificationHolder);
	}

	public void testSetActionHolder() {
	  	String path = (String)messageContext.getProperty( "resourceURI" );
//  	System.out.println(messageContext.getProperty( "resourceURI" ));
	  	URI resourceConfigPath = null;
		try {
			resourceConfigPath = new URI( path );
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
		path = (String)messageContext.getProperty( "sensorConfigURI" );
//  	System.out.println(messageContext.getProperty( "sensorConfigURI" ));
	  	URI resourcePath = null;
		try {
			resourcePath = new URI( path );
		} catch (URISyntaxException e1) {
			e1.printStackTrace();
		}
		ActionHolder actionHolder = new ActionHolder( messageContext, resourcePath, resourceConfigPath );
		notificationHolder.setActionHolder(actionHolder);
		// assert ...
	}

	public void testSetQueryProcessorHolder() {
		String path = (String)messageContext.getProperty( "resourceURI" );
//  	System.out.println(messageContext.getProperty( "resourceURI" ));
	  	URI resourceConfigPath = null;
		try {
			resourceConfigPath = new URI( path );
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
		path = (String)messageContext.getProperty( "sensorConfigURI" );
//  	System.out.println(messageContext.getProperty( "sensorConfigURI" ));
	  	URI resourcePath = null;
		try {
			resourcePath = new URI( path );
		} catch (URISyntaxException e1) {
			e1.printStackTrace();
		}
		
		QueryProcessorHolder queryProcessorHolder = new QueryProcessorHolder(messageContext,resourcePath,resourceConfigPath);
		notificationHolder.setQueryProcessorHolder(queryProcessorHolder);
		// assert ...
	
	}

	public void testGetRate() {
		int rate = notificationHolder.getRate();
		assertEquals(1000,rate);
	}

	public void testGetSubscriptionCount() {
		int subscriptionCount = notificationHolder.getSubscriptionCount();
		assertEquals(0,subscriptionCount);
	}

	public void testAddSubscription() {
		// subscription null
		notificationHolder.addSubscription(null);
		//
		SubscriptionPersistent subscriptionPersistent = new SubscriptionPersistent();
		notificationHolder.addSubscription(subscriptionPersistent);
		int subscriptionCount = notificationHolder.getSubscriptionCount();
		assertEquals(1,subscriptionCount);
	}

	public void testRemoveSubscription() {
		// subscription null
		notificationHolder.removeSubscription(null);
		//
		SubscriptionPersistent subscriptionPersistent = new SubscriptionPersistent();
		notificationHolder.removeSubscription(subscriptionPersistent);
		int subscriptionCount = notificationHolder.getSubscriptionCount();
		assertEquals(0,subscriptionCount);
		
	}
	

	public void testRun() {
		  Topic topic = new Topic();
	      topic.setName("TEST");
	      Dialect dialect = new Dialect();
			dialect.setName("Test dialect");
			Dialect[] dialects = new Dialect[1];
			dialects[0]=dialect;
			topic.setDialect(dialects);
	      TestSensor testSensor = new TestSensor();
	      testSensor.setName("Test Sensor");
	      try {
			testSensor.init();
		} catch (SensorException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}
	      TestSensorEvent testSensorEvent = new TestSensorEvent(testSensor,System.currentTimeMillis(),new String[]{"Test Event"});
	      testSensorEvent.setID(1);
	      testSensorEvent.setProducer("Test Sensor");
	      try {
			testSensorEvent.applyFormat(new TestFormat("Test Dialect"));
		} catch (SensorException e1) {
			e1.printStackTrace();
		}
	      topicHolder.addTopic(topic);
	      // set current event
	      topicHolder.setCurrentEvent(topic,testSensorEvent);
			notificationHolder = new NotificationHolder(1000,topicHolder,subscriptionRegistry);
		SubscriptionPersistent subscriptionPersistent = new SubscriptionPersistent();
		try {
			subscriptionPersistent.setMonitorConsumerURL(new org.apache.axis.types.URI("http://lxgrid04.pd.infn.it:8080/axis/services/CEMonitorConsumerPort"));
		} catch (MalformedURIException e) {
			e.printStackTrace();
		}
	      subscriptionPersistent.setTopic(topic);
	      Policy policy = new Policy();
	      policy.setRate(5000);
	      Query query = new Query();
	      query.setExpression("test");
	      query.setQueryLanguage("test language");
	      policy.setQuery(query);
	      subscriptionPersistent.setPolicy(policy);
		notificationHolder.addSubscription(subscriptionPersistent);

		String path = (String)messageContext.getProperty( "resourceURI" );
//  	System.out.println(messageContext.getProperty( "resourceURI" ));
	  	URI resourceConfigPath = null;
		try {
			resourceConfigPath = new URI( path );
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
		path = (String)messageContext.getProperty( "sensorConfigURI" );
//  	System.out.println(messageContext.getProperty( "sensorConfigURI" ));
	  	URI resourcePath = null;
		try {
			resourcePath = new URI( path );
		} catch (URISyntaxException e1) {
			e1.printStackTrace();
		}
		QueryProcessorHolder queryProcessorHolder = new QueryProcessorHolder(messageContext,resourcePath,resourceConfigPath);
		ClassAdProcessor queryProcessor = new ClassAdProcessor();
		queryProcessor.setId(1);
		queryProcessor.setName("test language");
		queryProcessorHolder.addResource(queryProcessor);
		notificationHolder.setQueryProcessorHolder(queryProcessorHolder);
		// assert ...
		notificationHolder.run();
	}


}
