/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */

import org.glite.ce.monitor.queryprocessor.RegExProcessor;
import org.glite.ce.monitorapij.queryprocessor.QueryResult;
import org.glite.ce.monitorapij.sensor.SensorException;
import org.glite.ce.monitorapij.types.Query;

import junit.framework.TestCase;
/**
 * @author borgia
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class RegExProcessorTest extends TestCase {

    private RegExProcessor regExProcessor = null;
    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        regExProcessor = new RegExProcessor();
    }

    public void testRegExProcessor() {
        assertNotNull(regExProcessor);
    }

    public void testEvaluate() {
        String exception = "";
        TestSensor testSensor = new TestSensor();
        testSensor.addFormat(new TestFormat("test format"));
	    testSensor.setName("Test Sensor");
		TestSensorEvent testSensorEvent = new TestSensorEvent( testSensor, System.currentTimeMillis( ), new String[]{"Test Event"} );
	      
		Query query = new Query();
        QueryResult[] queryResults =null; 
        // query null
        try {
            queryResults = regExProcessor.evaluate(null,testSensorEvent);
        } catch (Exception e) {
            exception = e.getMessage();
        }
        assertEquals("Invalid argument: the query is null!",exception);
        
        // query language null
        try {
            queryResults = regExProcessor.evaluate(query,testSensorEvent);
        } catch (Exception e) {
            exception = e.getMessage();
        }
        assertEquals("the query language is null",exception);
        

        // query language differret from RegEx
	      query.setQueryLanguage("nothing");
	      try {
	            queryResults = regExProcessor.evaluate(query,testSensorEvent);
	        } catch (Exception e) {
	            exception = e.getMessage();
	        }
	        assertEquals("Query Language mismatch: expected \"RegEx\", found \"nothing\"",exception);
	        
	        

		  query.setQueryLanguage("RegEx");
	      // null expression
		  try {
	            queryResults = regExProcessor.evaluate(query,testSensorEvent);
	        } catch (Exception e) {
	            exception = e.getMessage();
	        }
	        assertEquals("the query expression is null",exception);
	        
	        query.setExpression("Test");
	        // no exceptions about query
	        // event null
	        try {
	            queryResults = regExProcessor.evaluate(query,null);
	        } catch (Exception e) {
//	            e.printStackTrace();
	            exception = e.getMessage();
	        }
	        assertEquals("Invalid argument: the event is null!",exception);
	        
	        // event empty
	        try {
	            queryResults = regExProcessor.evaluate(query,testSensorEvent);
	        } catch (Exception e) {
//	            e.printStackTrace();
	            exception = e.getMessage();
	        }
	        assertEquals("the event is empty",exception);
	        
	        // no exceptions
			testSensorEvent.addParameter("first","first test");
			testSensorEvent.addParameter("second","second test");
			
	        
			try {
                testSensorEvent.applyFormat("test format");
            } catch (SensorException e1) {
                e1.printStackTrace();
            }

			try {
	            queryResults = regExProcessor.evaluate(query,testSensorEvent);
	        } catch (Exception e) {
	            e.printStackTrace();
	            exception = e.getMessage();
	        }
	        assertNotNull(queryResults);
	        assertEquals("Test Event",queryResults[0].getMessage());
    }

}
