/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Antonino Stefano Borgia <stefano.borgia@pd.infn.it>
 *
 */

import org.glite.ce.monitor.holder.TopicHolder;
import org.glite.ce.monitorapij.sensor.SensorEvent;
import org.glite.ce.monitorapij.types.Event;
import org.glite.ce.monitorapij.types.Topic;

import junit.framework.TestCase;


public class TopicHolderTest extends TestCase {
   private static TopicHolder topicHolder;
   private static Topic topic;
   private static Topic anotherTopic;
   private static TestSensorEvent testSensorEvent;
   
   public static void main( String[] args ) {
      junit.textui.TestRunner.run( TopicHolderTest.class );
   }


   public void testTopicHolder( ) {
      topicHolder = new TopicHolder();
      assertNotNull(topicHolder);
   }


   public void testAddTopic( ) {
      // case topic = null;
      topic = null;
      topicHolder.addTopic(topic);
      
      // adding a topic
      topic = new Topic();
      topic.setName("Test Topic");
      topicHolder.addTopic(topic);
      assertEquals("Test Topic",topic.getName());
      
   }


   /*
    * Class under test for void removeTopic(Topic)
    */
   public void testRemoveTopicTopic( ) {
      // case topic = null
      topic = null;
      topicHolder.removeTopic(topic);
      
      // case String topic = null
      String topicString = null;
      topicHolder.removeTopic(topicString);
      
      // adding 2 new topics, the first named "Test Topic", the second "Another Test Topic". 
      topic = new Topic();
      topic.setName("Test Topic");
      topicHolder.addTopic(topic);
      anotherTopic = new Topic();
      anotherTopic.setName("Another Test Topic");
      topicHolder.addTopic(anotherTopic);
      // verifies two topics has been added
      Topic[] topics;
      topics = topicHolder.getTopics();
      assertEquals(2,topics.length);
      // removes the first topic using topic name
      topicHolder.removeTopic("Test Topic");
      // verifies that only the "Another Test Topic" topic has remained
      topics = topicHolder.getTopics();
      assertEquals(1,topics.length);
      assertEquals("Another Test Topic",topics[0].getName());
      // removes the other topic using topic class
      topicHolder.removeTopic(anotherTopic);
      //    verifies that any topic has remained
      topics = topicHolder.getTopics();
      assertEquals(0,topics.length);
      
      
   }


   

   public void testCheckSupportedTopic( ) {
      // case topic = null
      topic = null;
      topicHolder.checkSupportedTopic(topic);
      
      //    adding 2 new topics, the first named "Test Topic", the second "Another Test Topic". 
      topic = new Topic();
      topic.setName("Test Topic");
      topicHolder.addTopic(topic);
      anotherTopic = new Topic();
      anotherTopic.setName("Another Test Topic");
      topicHolder.addTopic(anotherTopic);
      // verifying that the two topics are supported
      assertTrue(topicHolder.checkSupportedTopic(topic));
      assertTrue(topicHolder.checkSupportedTopic(anotherTopic));
      // removes the first topic using topic name
      topicHolder.removeTopic("Test Topic");
      // verifying that topic ins't anymore supported
      assertFalse(topicHolder.checkSupportedTopic(topic));
      
   }


   public void testSetCurrentEvent( ) {
      //    adding a topic
      topic = new Topic();
      topic.setName("Test Topic");
      topicHolder.addTopic(topic);
      // create a sensor and an event
      TestSensor testSensor = new TestSensor();
      testSensor.setName("Test Sensor");
      testSensorEvent = new TestSensorEvent(testSensor,System.currentTimeMillis(), new String[]{"Test Event"});
      testSensorEvent.setID(1);
      testSensorEvent.setProducer("Test Sensor");
      // set current event
      topicHolder.setCurrentEvent(topic,testSensorEvent);
      // verify current event ID: create an event with ID 0, then switch to current event and verify
      // its ID is now 1
      SensorEvent anotherSensorEvent = new SensorEvent(testSensor,0);
      assertEquals(0,anotherSensorEvent.getID());
      // get current event
      anotherSensorEvent = (SensorEvent) topicHolder.getCurrentEvent(topic);      
      assertEquals(1,anotherSensorEvent.getID());
      // verify current event producer
      assertEquals("Test Sensor",testSensorEvent.getProducer());
      
   }
   
   public void testGetCurrentEventString(){
      // setting a new event
      TestSensor testSensor = new TestSensor();
      testSensor.setName("Test Sensor");
      testSensorEvent = new TestSensorEvent(testSensor,System.currentTimeMillis(),new String[]{"Test Event"});
      testSensorEvent.setID(1);
      testSensorEvent.setProducer("Test Sensor");
      topicHolder.setCurrentEvent("Test Sensor",testSensorEvent);
      // get current event
      SensorEvent anotherSensorEvent = new SensorEvent(testSensor,0);
      anotherSensorEvent = (SensorEvent) topicHolder.getCurrentEvent("Test Topic");
      assertEquals(1,anotherSensorEvent.getID());
      
   }
   
   public void testGetCurrentEventTopic(){
      // case topic is null
      topic = null;
      Event event = null;
      event = topicHolder.getCurrentEvent(topic);
      
      // topic not null
      topic = new Topic();
      topic.setName("Test Topic");
      
      event = topicHolder.getCurrentEvent(topic);
      assertEquals(1,event.getID());
   }



   public void testGetTopics( ) {
      topic = new Topic();
      topic.setName("Test Topic");
      topicHolder = new TopicHolder();
      topicHolder.addTopic(topic);
      anotherTopic = new Topic();
      anotherTopic.setName("Another Test Topic");
      topicHolder.addTopic(anotherTopic);
      Topic[] topics;
      topics = topicHolder.getTopics();
      for (int i=0; i < topics.length; i++ ){
         assertNotNull(topics[i]);
         
      }
      assertEquals(2,topics.length);
      assertEquals("Test Topic",topics[1].getName());
      assertEquals("Another Test Topic",topics[0].getName());
      
   }
   
   public void testGetTopic( ) {
   	topic = new Topic();
    topic.setName("Test Topic");
    topicHolder = new TopicHolder();
    topicHolder.addTopic(topic);
    anotherTopic = new Topic();
    // not existing topic
    anotherTopic = topicHolder.getTopic("Not Existing");
    assertNull(anotherTopic);
    
    anotherTopic = topicHolder.getTopic("Test Topic");
    assertEquals("Test Topic",anotherTopic.getName());
    
 }

}
